/*
 *     This file is part of the Action Word Workspace plugin for Squash TM (henix-tm-workspace-action-word) project.
 *     Copyright (C) 2020 - 2021 Henix, henix.fr - All Rights Reserved
 *
 *     Unauthorized copying of this file, via any medium is strictly prohibited
 *     Proprietary and confidential
 *
 * 	 (C)Henix. Tous droits réservés.
 *
 * 	Avertissement : ce programme est protégé par la loi relative au droit d'auteur et par les conventions internationales. Toute reproduction ou distribution partielle ou totale du logiciel, par quelque moyen que ce soit, est strictement interdite.
 */
define(["jquery", "backbone", "tree", "underscore", "app/ws/squashtm.notification", "./permissions-rules",
        "workspace.contextual-content", "workspace.event-bus", "workspace.routing", "squash.translator",
        "workspace.tree-node-copier", "workspace.storage", "workspace.tree-event-handler", "user-account/user-prefs",
        "jquery.squash.messagedialog"],
    function ($, Backbone, zetree, _, notification, rules, ctxcontent, eventBus, urlBuilder, translator, copier, storage, treehandler, userPrefs) {

    function showError(messageName) {

        var messages = translator.get({
            "no-libraries-allowed": "tree.button.copy-node.error.nolibrary",
            "not-unique": "tree.button.copy-node.error.notOneEditable",
            "not-creatable": "tree.button.copy-node.error.notOneEditable",
            "empty-selection": "tree.button.copy-node.error.nothing-to-paste",
            "invalid-content": "tree.button.copy-node.error.pastenothere",
            "not-deletable": "dialog.label.delete-node.rejected",
            "milestone-denied": "squashtm.action.exception.milestonelocked",
            "duplicate-copy": "dialog.action-word.copy.duplicated"
        });

        squashtm.notification.showInfo(messages[messageName]);
    }

    function copyIfOk(tree) {
        var nodes = tree.jstree("get_selected");
        if (rules.canCopy(nodes)) {
            copier.copyNodesToCookie();
        } else {
            var why = rules.whyCantCopy(nodes);
            showError(why);
        }
    }

    function pasteIfOk() {
        if (rules.canPaste()) {
            pasteActionWordNodesFromCookie();
        } else {
            var why = rules.whyCantPaste();
            showError(why);
        }
    }

    function pasteActionWordNodesFromCookie() {
        var tree = copier.tree;
        var data = retrieve();
        var target = tree.jstree('get_selected');
        checkDuplicateTokens(target, data.nodes)
            .then(function(noDuplicateTokenFound) {
                if (!noDuplicateTokenFound) {
                    showError("duplicate-copy");
                }
                copier.pasteNodesFromCookie();
            });
    };

    function retrieve() {
        var data = storage.get('squash-copy-nodes') || null;
        return JSON.parse(data);
    };

    function checkDuplicateTokens(target, nodes) {
        var url = target.getBrowserUrl() + '/drives/' + target.getResId() + '/content/simulate-new';
        var nodeIds = _.map(nodes, function(node) { return node.resid; });
        var params = {
            'nodeIds[]': nodeIds
        };
        return $.ajax({
            method: 'POST',
            url: url,
            data: params
        });
    };

    function loadFragment(tree) {
        var selected = tree.jstree("get_selected");

        switch (selected.length) {
            // nothing selected : nothing is displayed
            case 0 :
                ctxcontent.unload();
                break;
            // exactly one element is selected : display it
            case 1 :
               ctxcontent.loadWith(selected.getResourceUrl());
                break;
            default :
                break;
        }

    }

    return {
        init: function () {

            var tree = zetree.get();

            $("#generic-info-dialog").messageDialog();

            tree.on("select_node.jstree deselect_node.jstree", function () {
                loadFragment(tree);
            });

            // ************* creation ***************

            $("#new-folder-tree-button").on("click", function () {
                $("#add-folder-dialog").formDialog("open");
            });

            $("#new-action-word-tree-button").on("click", function () {
                $("#add-action-word-dialog").formDialog("open");
            });

            // *************** copy paste ****************

            $("#copy-node-tree-button").on("click", function () {
                copyIfOk(tree);
            });

            // issue 2762 : the events "copy.squashtree" and the native js event "copy" (also triggered using ctrl+c) would both fire this
            // handler. Its a bug of jquery, fixed in 1.9.
            // TODO : upgrade to jquery 1.9
            tree.on("copy.squashtree", function (evt) {
                if (evt.namespace === "squashtree") {
                    copyIfOk(tree);
                }
            });

            $("#paste-node-tree-button").on("click", function () {
                pasteIfOk(tree);
            });

            // issue 2762 : the events "paste.squashtree" and the native js event "paste" (also triggered using ctrl+v) would both fire this
            // handler. Its a bug of jquery, fixed in 1.9
            // TODO : upgrade to jquery 1.9
            tree.on("paste.squashtree", function (evt) {
                if (evt.namespace === "squashtree") {
                    pasteIfOk(tree);
                }
            });

            // ***************** deletion ********************

            function findTestStepsAssociatedToActionWord(nodes) {
                var ids = nodes.treeNode().all('getResId').join(',');
                var rawUrl = squashtm.app.contextRoot + "action-word-browser/content/{nodeIds}";
                var url = rawUrl.replace('{nodeIds}', ids) + '/get-associated-keyword-test-steps';
                return $.getJSON(url);
            }

            function getNotEmptyAssociatedTestSteps(data, testStepList) {
                $('.delete-node-dialog-details').show();
                var actionWord = translator.get("dialog.label.delete-action-word.action-word");
                var currentlyInUse = translator.get("dialog.label.delete-action-word.currently-in-use");
                var keywordTestSteps = translator.get("dialog.label.delete-action-word.keyword-test-steps");
                $.each(data, function (key, value) {
                    var text = actionWord + " <" + key + "> " + currentlyInUse + " <strong>" + value + "</strong> " + keywordTestSteps;
                    var li = $('<li/>')
                        .addClass('ui-menu-item')
                        .attr('role', 'menuitem')
                        .appendTo(testStepList);
                    var span1 = $('<span/>')
                        .text(actionWord + " <" + key + "> " + currentlyInUse)
                        .appendTo(li);
                    var span2 = $('<span/>')
                        .addClass('bold-warning-message')
                        .text(" "+value+" ")
                        .appendTo(li);
                    var span3 = $('<span/>')
                        .text(keywordTestSteps)
                        .appendTo(li);
                });
            }

            function getAssociatedTestSteps(data) {
                var testStepList = $("#test-step-list");
                testStepList.empty();
                if (!$.isEmptyObject(data)) {
                    getNotEmptyAssociatedTestSteps(data, testStepList);
                } else {
                    $('.delete-node-dialog-details').hide();
                }
            }

            function checkNodeDeletable() {
                var nodes = tree.jstree('get_selected');
                if (!rules.canDelete(nodes)) {
                    showError(rules.whyCantDelete(nodes));
                } else {
                    findTestStepsAssociatedToActionWord(nodes).then(function(data) {
                        if (nodes.length === Object.keys(data).length) {
                            notification.showInfo(translator.get("dialog.label.delete-selected-action-word.all-associated-to-steps"));
                        } else {
                            getAssociatedTestSteps(data);
                            $("#delete-action-word-dialog").delnodeDialog("open");
                        }
                    });
                }
            }

            $("#delete-node-tree-button").on("click", checkNodeDeletable);

            tree.on("suppr.squashtree", checkNodeDeletable);





        }
    };

});
