/*
 *     This file is part of the Action Word Workspace plugin for Squash TM (henix-tm-workspace-action-word) project.
 *     Copyright (C) 2020 - 2021 Henix, henix.fr - All Rights Reserved
 *
 *     Unauthorized copying of this file, via any medium is strictly prohibited
 *     Proprietary and confidential
 *
 * 	 (C)Henix. Tous droits réservés.
 *
 * 	Avertissement : ce programme est protégé par la loi relative au droit d'auteur et par les conventions internationales. Toute reproduction ou distribution partielle ou totale du logiciel, par quelque moyen que ce soit, est strictement interdite.
 */
define(['jquery', 'workspace.tree-node-copier'],
    function ($, copier) {

    squashtm = squashtm || {};
    squashtm.workspace = squashtm.workspace || {};

    if (squashtm.workspace.permissions_rules === undefined) {
        squashtm.workspace.permissions_rules = new ActionWordPermissionsRules();
    }

    return squashtm.workspace.permissions_rules;

    /* Main Object */
    function ActionWordPermissionsRules() {

        this.canCreateButton = function (nodes) {
            return nodes.filter(':creatable').length === 1;
        };

        this.canCreateActionWord = function (nodes) {
            return nodes.filter(':creatable').length === 1;
        };

        this.whyCantCreate = function (nodes) {
            if (!this.canCreateButton(nodes)) {
                return "permission-denied";
            } else {
                return "yes-you-can";
            }
        };

        // must be not empty, and not contain libraries.
        this.canCopy = function (nodes) {
            return (nodes.length > 0) && (!nodes.is(':library'));
        };

        this.whyCantCopy = function (nodes) {
            if (nodes.length === 0) {
                return "empty-selection";
            }
            if (nodes.is(':library')) {
                return "no-libraries-allowed";
            }
            return "yes-you-can";
        };

        this.whyCantPaste = function () {

            var nodes = copier.bufferedNodes();

            if (nodes.length === 0) {
                return "empty-selection";
            }

            var target = nodes.tree.get_selected();

            if (target.length !== 1) {
                return "not-unique";
            }

            if (!target.isCreatable()) {
                return "not-creatable";
            }

            if (!target.acceptsAsContent(nodes)) {
                return 'invalid-content';
            }

            return "yes-you-can";
        };

        this.canPaste = $.proxy(function (nodes) {
            return (this.whyCantPaste(nodes) === "yes-you-can");
        }, this);

        this.canDelete = function (nodes) {
            return (nodes.filter(':deletable').not(':library').length == nodes.length) && (nodes.length > 0);
        };

        this.whyCantDelete = function (nodes) {
            if (nodes.length === 0) {
                return "empty-selection";
            }

            if (nodes.not(':deletable').length > 0) {
                return "not-deletable";
            }

            if (nodes.is(':library')) {
                return "no-libraries-allowed";
            }

            return "yes-you-can";
        };

        this.canDnD = function (movednodes, newparent) {
            // check if the node is draggable first
            if (movednodes.is(':library')) {
                return false;
            }
            //check that moving the node will not remove it from its original container
            if (!squashtm.keyEventListener.ctrl && !movednodes.isDeletable()) {
                return false;
            }
            // check that the destination type is legal
            if (!newparent.isCreatable() || !newparent.acceptsAsContent(movednodes)) {
                return false;
            }

            return true;
        };

        this.buttonrules = {
            'tree-create-button': this.canCreateButton,
            'new-action-word-tree-button': this.canCreateActionWord,
            'copy-node-tree-button': this.canCopy,
            'paste-node-tree-button': this.canPaste,
            'delete-node-tree-button': this.canDelete
        };

    }


});
