/*
 *     This file is part of the Action Word Workspace plugin for Squash TM (henix-tm-workspace-action-word) project.
 *     Copyright (C) 2020 - 2021 Henix, henix.fr - All Rights Reserved
 *
 *     Unauthorized copying of this file, via any medium is strictly prohibited
 *     Proprietary and confidential
 *
 * 	 (C)Henix. Tous droits réservés.
 *
 * 	Avertissement : ce programme est protégé par la loi relative au droit d'auteur et par les conventions internationales. Toute reproduction ou distribution partielle ou totale du logiciel, par quelque moyen que ce soit, est strictement interdite.
 */
define(["jquery", "backbone", "underscore", "squash.translator", "jeditable.simpleJEditable", "app/ws/squashtm.notification", "squashtable"],
    function($, Backbone, _, translator, SimpleJEditable, notification) {

        var ParameterTable = Backbone.View.extend({

            el: "#parameters-table",

            initialize: function(settings) {
                _.bindAll(this, "configureSimpleJEditables");
                this.settings = settings;
                this._configureTable(settings);
            },

            _configureTable: function(settings) {
                this.$el.squashTable(this._dataTableSettings(settings), {});
                this.table = this.$el.squashTable();
            },
            _dataTableSettings: function(settings) {
                var fnRowCallback = undefined;
                if(settings.actionWord.writable) {
                    fnRowCallback = this.configureSimpleJEditables;
                }
                return {
                    "bPaginate" : false,
                    "fnRowCallback": fnRowCallback
                };
            },
            configureSimpleJEditables: function(row, data) {
                var self = this;
                var actionWordId = self.settings.actionWord.id;
                var postUrlBase = squashtm.app.contextRoot + 'action-word/' + actionWordId + '/parameters/' + data['entity-id'];
                this.configureJEditableForName(postUrlBase, row, data);
                this.configureJEditableForDefaultValue(postUrlBase, row, data);
            },
            configureJEditableForName: function(postUrlBase, row, data) {
                var self = this;
                var postUrl = postUrlBase + "/name";
                var component = $('td.parameter-name', row);
                var validateNameModification = function(value, settings) {
                    if(settings.oldName === undefined) {
                        settings.oldName = data['name'];
                    }
                    var pattern = /^[A-Za-z0-9_\-]{1,255}$/;
                    if(!pattern.test(value)) {
                        notification.showError(translator.get("message.parameterInvalidPattern"));
                        return settings.oldName;
                    } else {
                        $.ajax({
                            url: postUrl,
                            data: { "value": value },
                            method: "POST"
                        });
                        settings.oldName = value;
                        return value;
                    }
                };
                new SimpleJEditable({
                    targetUrl:  validateNameModification,
                    component: component,
                    jeditableSettings: {
                        callback: function(newName) {
                            self.trigger("actionword.parameter.name.update",{
                                id : data['entity-id'],
                                name : newName
                            });
                        },
                        onerror: function(settings, original, xhr) {
                            console.log(original);
                            xhr.errorIsHandled = true;
                            notification.showXhrInDialog(xhr);
                            original.reset(this);
                        }
                    }
                });

            },
            configureJEditableForDefaultValue: function(postUrlBase, row, data) {
                var self = this;
                var postUrl = postUrlBase + "/default-value";
                var component = $('td.parameter-default-value', row);
                var validateDefaultValueModification = function(value, settings) {
                    if(settings.oldDefaultValue === undefined) {
                        settings.oldDefaultValue = data['default-value'];
                    }
                    var pattern = /^[^<>"]{0,255}$/;
                    if(!pattern.test(value)) {
                        notification.showError(translator.get('error.message.action-word.default-value.invalid'));
                        return settings.oldDefaultValue;
                    } else {
                        $.ajax({
                            url: postUrl,
                            data: { "value": value },
                            method: "POST"
                        });
                        var trimmedValue = value.trim();
                        var removedExtraSpacesValue = trimmedValue.replace(/\s+/g,' ');
                        settings.oldDefaultValue = removedExtraSpacesValue;
                        return removedExtraSpacesValue;
                    }
                };
                new SimpleJEditable({
                    targetUrl: validateDefaultValueModification,
                    component: component,
                    jeditableSettings: {
                        callback: function(newName) {
                            self.trigger("actionword.parameter.default-value.update",{
                                id : data['entity-id'],
                                name : newName
                            });
                        },
                        onerror: function (settings, original, xhr) {
                            console.log(original);
                            xhr.errorIsHandled = true;
                            notification.showXhrInDialog(xhr);
                            original.reset(this);
                        }
                    }
                });
            }
        });

        return ParameterTable;
});
